from .domain import HealthCheckInterface
from .model import HealthCheckModel, HealthCheckEntityModel
from .enum import HealthCheckStatusEnum
from typing import List
from datetime import datetime


class HealthCheckFactory:
    _healthItems: List[HealthCheckInterface]
    _health: HealthCheckModel

    def __init__(self) -> None:
        self._healthItems = list()

    def add(self, item: HealthCheckInterface) -> None:
        self._healthItems.append(item)

    def __startTimer__(self, entityTimer: bool) -> None:
        if entityTimer == True:
            self._entityStartTime = datetime.now()
        else:
            self._totalStartTime = datetime.now()

    def __stopTimer__(self, entityTimer: bool) -> None:
        if entityTimer == True:
            self._entityStopTime = datetime.now()
        else:
            self._totalStopTime = datetime.now()

    def __getTimeTaken__(self, entityTimer: bool) -> datetime:
        if entityTimer == True:
            return self._entityStopTime - self._entityStartTime
        return self._totalStopTime - self._totalStartTime

    def __dumpModel__(self, model: HealthCheckModel) -> str:
        """This goes and convert python objects to something a json object."""
        l = list()
        for i in model.entities:
            i.status = i.status.value
            i.timeTaken = str(i.timeTaken)
            l.append(dict(i))

        model.entities = l
        model.status = model.status.value
        model.totalTimeTaken = str(model.totalTimeTaken)

        return dict(model)

    def check(self) -> HealthCheckModel:
        self._health = HealthCheckModel()
        self.__startTimer__(False)
        for i in self._healthItems:
            # Generate the model
            item = HealthCheckEntityModel(alias=i._alias, tags=i._tags)

            # Track how long the entity took to respond
            self.__startTimer__(True)
            item.status = i.__checkHealth__()
            self.__stopTimer__(True)
            item.timeTaken = self.__getTimeTaken__(True)

            # if we have one dependency unhealthy, the service in unhealthy
            if item.status == HealthCheckStatusEnum.UNHEALTHY:
                self._health.status = HealthCheckStatusEnum.UNHEALTHY

            self._health.entities.append(item)
        self.__stopTimer__(False)
        self._health.totalTimeTaken = self.__getTimeTaken__(False)

        self._health = self.__dumpModel__(self._health)

        return self._health


class HealthCheckBase:
    def setConnectionUri(self, value: str) -> None:
        if value == "":
            raise Exception(f"{self._service} ConnectionUri is missing a value.")
        self._connectionUri = value

    def getConnectionUri(self) -> str:
        return self._connectionUri

    def setName(self, value: str) -> str:
        if not value:
            raise Exception("Missing a valid name.")
        self._name = value

    def getService(self) -> str:
        return self._service

    def getTags(self) -> List[str]:
        return self._tags

    def getAlias(self) -> str:
        return self._alias



from typing import Callable
from starlette.responses import JSONResponse
from .service import HealthCheckFactory
from .enum import HealthCheckStatusEnum


def healthCheckRoute(factory: HealthCheckFactory) -> Callable:
    """
    This function is passed to the add_api_route with the built factory.

    When called, the endpoint method within, will be called and it will run the job bound to the factory.
    The results will be parsed and sent back to the requestor via JSON.
    """
    
    _factory = factory

    def endpoint() -> JSONResponse:
        res = _factory.check()    
        if res['status'] == HealthCheckStatusEnum.UNHEALTHY.value:
            return JSONResponse(content=res, status_code=500)
        return JSONResponse(content=res, status_code=200)

    return endpoint




from datetime import datetime
from typing import List, Optional, Union
from .enum import HealthCheckStatusEnum
from datetime import datetime
from pydantic import BaseModel


class HealthCheckEntityModel(BaseModel):
    alias: str
    status: Union[HealthCheckStatusEnum, str] = HealthCheckStatusEnum.HEALTHY
    timeTaken: Union[Optional[datetime], str]
    tags: List[str] = list()


class HealthCheckModel(BaseModel):
    status: Union[HealthCheckStatusEnum, str] = HealthCheckStatusEnum.HEALTHY
    totalTimeTaken: Union[Optional[datetime], str]
    entities: List[HealthCheckEntityModel] = list()



from abc import ABC, abstractclassmethod
from typing import List
from .enum import HealthCheckStatusEnum


class HealthCheckInterface(ABC):
    _connectionUri: str
    _alias: str
    _tags: List[str]

    @abstractclassmethod
    def setConnectionUri(self, value: str) -> None:
        """ConnectionUri will be the value that is requested to check the health of an endpoint."""
        pass

    @abstractclassmethod
    def setName(self, value: str) -> None:
        """The Name is the friendly name of the health object."""
        pass

    @abstractclassmethod
    def getService(self) -> str:
        """The Service is a definition of what kind of endpoint we are checking on."""
        pass

    @abstractclassmethod
    def getTags(self) -> List[str]:
        pass

    @abstractclassmethod
    def __checkHealth__(self) -> HealthCheckStatusEnum:
        """Requests data from the endpoint to validate health."""
        pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fastapi_healthcheck']

package_data = \
{'': ['*']}

install_requires = \
['fastapi>=0,<1']

setup_kwargs = {
    'name': 'fastapi-healthcheck',
    'version': '0.2.12',
    'description': 'Base package to handle health checks with FastAPI.',
    'long_description': '# fastapi_healthcheck\n\nEasy to use health check for your FastAPI.  This is the root module that will let you add implement and expand your usage of health checks, with FastAPI.\n\nThis module does not contain any service checkers, but you can easily add them.  The other modules are not in this root module due to different dependencies required for each one.  This is made so you only bring in the packages that you need to not add extra packages.\n\n## Install\n\n`pip install fastapi-healthcheck` or `poetry add fastapi-healthcheck`\n\n## Adding Health Checks\n\nHere is what you need to get started.\n\n```python\nfrom fastapi import FastAPI\nfrom fastapi_healthcheck import HealthCheckFactory, healthCheckRoute\nfrom fastapi_healthcheck_sqlalchemy import HealthCheckSQLAlchemy\n\napp = FastAPI()\n\n# Add Health Checks\n_healthChecks = HealthCheckFactory()\n\n# SQLAlchemy comes from fastapi-healthcheck-sqlalchemy\n_healthChecks.add(HealthCheckSQLAlchemy(alias=\'postgres db\', connectionUri=cs.value, table=SmtpContactsSqlModel, tags=(\'postgres\', \'db\', \'sql01\')))\n\n# This will check external URI and validate the response that is returned.\n# fastapi-healthcheck-uri\n_healthChecks.add(HealthCheckUri(alias=\'reddit\', connectionUri="https://www.reddit.com/r/aww.json", tags=(\'external\', \'reddit\', \'aww\')))\napp.add_api_route(\'/health\', endpoint=healthCheckRoute(factory=_healthChecks))\n\n```\n\n## Returned Data\n\nWhen you request your health check, it will go and check all the entries that have been submitted and run a basic query against them.  If they come back as expected, then a status code is 200.  But if it runs into an error, it will return a 500 error.\n\n```json\n\n{\n    "status":"Healthy",\n    "totalTimeTaken":"0:00:00.671642",\n    "entities":[\n        {\n            "alias":"postgres db",\n            "status":"Healthy",\n            "timeTaken":"0:00:00.009619",\n            "tags":["postgres","db","sql01"]\n        },\n        {\n            "alias":"reddit",\n            "status":"Unhealthy",\n            "timeTaken":"0:00:00.661716",\n            "tags":["external","reddit","aww"]\n        }\n    ]\n}\n```\n\n## Available Modules\n\n* [fastapi_healthcheck_sqlalchemy](https://github.com/jtom38/fastapi_healthcheck_sqlalchemy)\n* [fastapi_healthcheck_uri](https://github.com/jtom38/fastapi_healthcheck_uri)\n\nIf you have made a public service module and want to see it on this list, please open a new issue so we can add it to the list.\n\n## Writing a custom module\n\nYou can easily expand on this core module to add other health checks for other services.  Generate a new service that pulls in [HealthCheckInterface](https://github.com/jtom38/fastapi_healthcheck/blob/master/fastapi_healthcheck/domain.py#L6) and [HealthCheckBase](https://github.com/jtom38/fastapi_healthcheck/blob/master/fastapi_healthcheck/service.py#L75).  With those, you can build the respective class around the interface.\n\nOnce you have your service ready to go, add it to the HealthCheckFactory, and let the testing start.\n\nIf you would like to see an example of a custom service see [fastapi_healthcheck_sqlalchemy](https://github.com/jtom38/fastapi_healthcheck_sqlalchemy/blob/master/fastapi_healthcheck_sqlalchemy/service.py).  This will give you a better example of what you need to do to create your own module to interface with healthcheck.\n',
    'author': 'James Tombleson',
    'author_email': 'luther38@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/jtom38/fastapi_healthcheck',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
